@@ -15,7 +15,8 @@ FIXTURES_PATH = os.path.realpath(os.path.join(os.path.dirname(__file__), 'fixtur
 class DNSTest(unittest.TestCase):
 
     def setUp(self):
-        self.channel = pycares.Channel(timeout=5.0, tries=1)
+        self.channel = pycares.Channel(timeout=10.0, tries=1, servers=['8.8.8.8', '8.8.4.4'])
+        self.is_ci = os.environ.get('APPVEYOR') or os.environ.get('TRAVIS') or os.environ.get('GITHUB_ACTION')
 
     def tearDown(self):
         self.channel = None
@@ -36,10 +37,73 @@ class DNSTest(unittest.TestCase):
                 self.channel.process_fd(pycares.ARES_SOCKET_BAD, fd)
 
     def assertNoError(self, errorno):
-        if errorno == pycares.errno.ARES_ETIMEOUT and (os.environ.get('APPVEYOR') or os.environ.get('TRAVIS')):
+        if errorno == pycares.errno.ARES_ETIMEOUT and self.is_ci:
             raise unittest.SkipTest('timeout')
         self.assertEqual(errorno, None)
 
+    @unittest.skipIf(sys.platform == 'win32', 'skipped on Windows')
+    def test_getaddrinfo(self):
+        self.result, self.errorno = None, None
+        def cb(result, errorno):
+            self.result, self.errorno = result, errorno
+        self.channel.getaddrinfo('localhost', 80, cb)
+        self.wait()
+        self.assertNoError(self.errorno)
+        self.assertEqual(type(self.result), pycares.ares_addrinfo_result)
+        self.assertTrue(len(self.result.nodes) > 0)
+        for node in self.result.nodes:
+            self.assertEqual(node.addr[1], 80)
+
+    @unittest.skipIf(sys.platform == 'win32', 'skipped on Windows')
+    def test_getaddrinfo2(self):
+        self.result, self.errorno = None, None
+        def cb(result, errorno):
+            self.result, self.errorno = result, errorno
+        self.channel.getaddrinfo('localhost', 'http', cb)
+        self.wait()
+        self.assertNoError(self.errorno)
+        self.assertEqual(type(self.result), pycares.ares_addrinfo_result)
+        self.assertTrue(len(self.result.nodes) > 0)
+        for node in self.result.nodes:
+            self.assertEqual(node.addr[1], 80)
+
+    @unittest.skipIf(sys.platform == 'win32', 'skipped on Windows')
+    def test_getaddrinfo3(self):
+        self.result, self.errorno = None, None
+        def cb(result, errorno):
+            self.result, self.errorno = result, errorno
+        self.channel.getaddrinfo('localhost', None, cb)
+        self.wait()
+        self.assertNoError(self.errorno)
+        self.assertEqual(type(self.result), pycares.ares_addrinfo_result)
+        self.assertTrue(len(self.result.nodes) > 0)
+        for node in self.result.nodes:
+            self.assertEqual(node.addr[1], 0)
+
+    @unittest.skipIf(sys.platform == 'win32', 'skipped on Windows')
+    def test_getaddrinfo4(self):
+        self.result, self.errorno = None, None
+        def cb(result, errorno):
+            self.result, self.errorno = result, errorno
+        self.channel.getaddrinfo('localhost', 'http', cb, family=socket.AF_INET)
+        self.wait()
+        self.assertNoError(self.errorno)
+        self.assertEqual(type(self.result), pycares.ares_addrinfo_result)
+        self.assertEqual(len(self.result.nodes), 1)
+        node = self.result.nodes[0]
+        self.assertEqual(node.addr[0], b'127.0.0.1')
+        self.assertEqual(node.addr[1], 80)
+
+    def test_getaddrinfo5(self):
+        self.result, self.errorno = None, None
+        def cb(result, errorno):
+            self.result, self.errorno = result, errorno
+        self.channel.getaddrinfo('google.com', 'http', cb)
+        self.wait()
+        self.assertNoError(self.errorno)
+        self.assertEqual(type(self.result), pycares.ares_addrinfo_result)
+        self.assertTrue(len(self.result.nodes) > 0)
+
     @unittest.skipIf(sys.platform == 'win32', 'skipped on Windows')
     def test_gethostbyaddr(self):
         self.result, self.errorno = None, None
@@ -51,7 +115,6 @@ class DNSTest(unittest.TestCase):
         self.assertEqual(type(self.result), pycares.ares_host_result)
 
     @unittest.skipIf(sys.platform == 'win32', 'skipped on Windows')
-    @unittest.skipIf(os.environ.get('TRAVIS') is not None, 'skipped on Travis')
     def test_gethostbyaddr6(self):
         self.result, self.errorno = None, None
         def cb(result, errorno):
@@ -94,6 +157,32 @@ class DNSTest(unittest.TestCase):
         self.assertIn(self.result.node, ('localhost.localdomain', 'localhost'))
         self.assertEqual(self.result.service, 'http')
 
+    @unittest.skipIf(sys.platform == 'win32', 'skipped on Windows')
+    @unittest.expectedFailure  # c-ares is broken (does not return numeric service if asked) and unconditionally adds zero scope
+    def test_getnameinfo_ipv6(self):
+        self.result, self.errorno = None, None
+        def cb(result, errorno):
+            self.result, self.errorno = result, errorno
+        self.channel.getnameinfo(('fd01:dec0:0:1::2020', 80, 0, 0), pycares.ARES_NI_NUMERICHOST|pycares.ARES_NI_NUMERICSERV, cb)
+        self.wait()
+        self.assertNoError(self.errorno)
+        self.assertEqual(type(self.result), pycares.ares_nameinfo_result)
+        self.assertEqual(self.result.node, 'fd01:dec0:0:1::2020')
+        self.assertEqual(self.result.service, '80')
+
+    @unittest.skipIf(sys.platform == 'win32', 'skipped on Windows')
+    @unittest.expectedFailure  # c-ares is broken (does not return numeric service if asked)
+    def test_getnameinfo_ipv6_ll(self):
+        self.result, self.errorno = None, None
+        def cb(result, errorno):
+            self.result, self.errorno = result, errorno
+        self.channel.getnameinfo(('fe80::5abd:fee7:4177:60c0', 80, 0, 666), pycares.ARES_NI_NUMERICHOST|pycares.ARES_NI_NUMERICSERV|pycares.ARES_NI_NUMERICSCOPE, cb)
+        self.wait()
+        self.assertNoError(self.errorno)
+        self.assertEqual(type(self.result), pycares.ares_nameinfo_result)
+        self.assertEqual(self.result.node, 'fe80::5abd:fee7:4177:60c0%666')
+        self.assertEqual(self.result.service, '80')
+
     def test_query_a(self):
         self.result, self.errorno = None, None
         def cb(result, errorno):
@@ -104,7 +193,6 @@ class DNSTest(unittest.TestCase):
         for r in self.result:
             self.assertEqual(type(r), pycares.ares_query_a_result)
             self.assertNotEqual(r.host, None)
-            self.assertTrue(r.ttl >= 0)
 
     def test_query_a_bad(self):
         self.result, self.errorno = None, None
@@ -141,7 +229,17 @@ class DNSTest(unittest.TestCase):
         for r in self.result:
             self.assertEqual(type(r), pycares.ares_query_aaaa_result)
             self.assertNotEqual(r.host, None)
-            self.assertTrue(r.ttl >= 0)
+
+    def test_query_caa(self):
+        self.result, self.errorno = None, None
+        def cb(result, errorno):
+            self.result, self.errorno = result, errorno
+        self.channel.query('wikipedia.org', pycares.QUERY_TYPE_CAA, cb)
+        self.wait()
+        self.assertNoError(self.errorno)
+        self.assertTrue(len(self.result) > 0)
+        for r in self.result:
+            self.assertEqual(type(r), pycares.ares_query_caa_result)
 
     def test_query_cname(self):
         self.result, self.errorno = None, None
@@ -161,7 +259,6 @@ class DNSTest(unittest.TestCase):
         self.assertNoError(self.errorno)
         for r in self.result:
             self.assertEqual(type(r), pycares.ares_query_mx_result)
-            self.assertTrue(r.ttl >= 0)
 
     def test_query_ns(self):
         self.result, self.errorno = None, None
@@ -182,7 +279,6 @@ class DNSTest(unittest.TestCase):
         self.assertNoError(self.errorno)
         for r in self.result:
             self.assertEqual(type(r), pycares.ares_query_txt_result)
-            self.assertTrue(r.ttl >= 0)
 
     def test_query_txt_chunked(self):
         self.result, self.errorno = None, None
@@ -200,12 +296,22 @@ class DNSTest(unittest.TestCase):
         self.result, self.errorno = None, None
         def cb(result, errorno):
             self.result, self.errorno = result, errorno
-        self.channel.query('s-pulse.co.jp', pycares.QUERY_TYPE_TXT, cb)
+        self.channel.query('google.com', pycares.QUERY_TYPE_TXT, cb)
         self.wait()
         self.assertNoError(self.errorno)
-        # s-pulse.co.jp.      3600    IN  TXT "MS=ms18955624"
-        # s-pulse.co.jp.      3600    IN  TXT "v=spf1 " "include:spf-bma.mpme.jp ip4:202.248.11.9 ip4:202.248.11.10 " "ip4:218.223.68.132 ip4:218.223.68.77 ip4:210.254.139.121 " "ip4:211.128.73.121 ip4:210.254.139.122 ip4:211.128.73.122 " "ip4:210.254.139.123 ip4:211.128.73.123 ip4:210.254.139.124 " "ip4:211.128.73.124 ip4:210.254.139.13 ip4:211.128.73.13 " "ip4:52.68.199.198 include:spf.betrend.com " "include:spf.protection.outlook.com " "~all"
-        self.assertEqual(len(self.result), 2)
+        # > dig -t txt google.com
+        # google.com.		3270	IN	TXT	"google-site-verification=TV9-DBe4R80X4v0M4U_bd_J9cpOJM0nikft0jAgjmsQ"
+        # google.com.		3270	IN	TXT	"atlassian-domain-verification=5YjTmWmjI92ewqkx2oXmBaD60Td9zWon9r6eakvHX6B77zzkFQto8PQ9QsKnbf4I"
+        # google.com.		3270	IN	TXT	"docusign=05958488-4752-4ef2-95eb-aa7ba8a3bd0e"
+        # google.com.		3270	IN	TXT	"facebook-domain-verification=22rm551cu4k0ab0bxsw536tlds4h95"
+        # google.com.		3270	IN	TXT	"google-site-verification=wD8N7i1JTNTkezJ49swvWW48f8_9xveREV4oB-0Hf5o"
+        # google.com.		3270	IN	TXT	"apple-domain-verification=30afIBcvSuDV2PLX"
+        # google.com.		3270	IN	TXT	"webexdomainverification.8YX6G=6e6922db-e3e6-4a36-904e-a805c28087fa"
+        # google.com.		3270	IN	TXT	"MS=E4A68B9AB2BB9670BCE15412F62916164C0B20BB"
+        # google.com.		3270	IN	TXT	"v=spf1 include:_spf.google.com ~all"
+        # google.com.		3270	IN	TXT	"globalsign-smime-dv=CDYX+XFHUw2wml6/Gb8+59BsH31KzUr6c1l2BPvqKX8="
+        # google.com.		3270	IN	TXT	"docusign=1b0a6754-49b1-4db5-8540-d2c12664b289"
+        self.assertEqual(len(self.result), 11)
 
     def test_query_txt_bytes1(self):
         self.result, self.errorno = None, None
@@ -217,8 +323,12 @@ class DNSTest(unittest.TestCase):
         for r in self.result:
             self.assertEqual(type(r), pycares.ares_query_txt_result)
             self.assertIsInstance(r.text, str)  # it's ASCII
-            self.assertTrue(r.ttl >= 0)
 
+    # The 2 tests below hit a dead end thus fail. Commenting for now as I couldn't find a live server
+    # that satisfies what the tests are looking for
+
+    # FIXME: wide.com.es is a dead end!
+    @unittest.expectedFailure
     def test_query_txt_bytes2(self):
         self.result, self.errorno = None, None
         def cb(result, errorno):
@@ -229,7 +339,42 @@ class DNSTest(unittest.TestCase):
         for r in self.result:
             self.assertEqual(type(r), pycares.ares_query_txt_result)
             self.assertIsInstance(r.text, bytes)
-            self.assertTrue(r.ttl >= 0)
+
+    # FIXME: "txt-non-ascii.dns-test.hmnid.ru" is a dead end!
+    @unittest.expectedFailure
+    def test_query_txt_multiple_chunked_with_non_ascii_content(self):
+        self.result, self.errorno = None, None
+        def cb(result, errorno):
+            self.result, self.errorno = result, errorno
+        self.channel.query('txt-non-ascii.dns-test.hmnid.ru', pycares.QUERY_TYPE_TXT, cb)
+        self.wait()
+        self.assertNoError(self.errorno)
+        # txt-non-ascii.dns-test.hmnid.ru.        IN      TXT     "ascii string" "some\208misc\208stuff"
+
+        self.assertEqual(len(self.result), 1)
+        r = self.result[0]
+        self.assertEqual(type(r), pycares.ares_query_txt_result)
+        self.assertIsInstance(r.text, bytes)
+
+    def test_query_class_chaos(self):
+        self.result, self.errorno = None, None
+        def cb(result, errorno):
+            self.result, self.errorno = result, errorno
+
+        self.channel.servers = ['199.7.83.42']  # l.root-servers.net
+        self.channel.query('id.server', pycares.QUERY_TYPE_TXT, cb, pycares.QUERY_CLASS_CHAOS)
+        self.wait()
+        self.assertNoError(self.errorno)
+        # id.server.              0       CH      TXT     "aa.de-ham.l.root"
+
+        self.assertEqual(len(self.result), 1)
+        r = self.result[0]
+        self.assertEqual(type(r), pycares.ares_query_txt_result)
+        self.assertIsInstance(r.text, str)
+
+    def test_query_class_invalid(self):
+        self.assertRaises(ValueError, self.channel.query, 'google.com', pycares.QUERY_TYPE_A, lambda *x: None, "INVALIDTYPE")
+        self.wait()
 
     def test_query_soa(self):
         self.result, self.errorno = None, None
@@ -239,7 +384,6 @@ class DNSTest(unittest.TestCase):
         self.wait()
         self.assertNoError(self.errorno)
         self.assertEqual(type(self.result), pycares.ares_query_soa_result)
-        self.assertTrue(self.result.ttl >= 0)
 
     def test_query_srv(self):
         self.result, self.errorno = None, None
@@ -250,7 +394,6 @@ class DNSTest(unittest.TestCase):
         self.assertNoError(self.errorno)
         for r in self.result:
             self.assertEqual(type(r), pycares.ares_query_srv_result)
-            self.assertTrue(r.ttl >= 0)
 
     def test_query_naptr(self):
         self.result, self.errorno = None, None
@@ -261,7 +404,6 @@ class DNSTest(unittest.TestCase):
         self.assertNoError(self.errorno)
         for r in self.result:
             self.assertEqual(type(r), pycares.ares_query_naptr_result)
-            self.assertTrue(r.ttl >= 0)
 
     def test_query_ptr(self):
         self.result, self.errorno = None, None
@@ -272,9 +414,6 @@ class DNSTest(unittest.TestCase):
         self.wait()
         self.assertNoError(self.errorno)
         self.assertEqual(type(self.result), pycares.ares_query_ptr_result)
-        self.assertIsInstance(self.result.ttl, int)
-        self.assertGreaterEqual(self.result.ttl, 0)
-        self.assertLessEqual(self.result.ttl, 2**31-1)
         self.assertEqual(type(self.result.aliases), list)
 
     def test_query_ptr_ipv6(self):
@@ -286,11 +425,9 @@ class DNSTest(unittest.TestCase):
         self.wait()
         self.assertNoError(self.errorno)
         self.assertEqual(type(self.result), pycares.ares_query_ptr_result)
-        self.assertIsInstance(self.result.ttl, int)
-        self.assertGreaterEqual(self.result.ttl, 0)
-        self.assertLessEqual(self.result.ttl, 2**31-1)
         self.assertEqual(type(self.result.aliases), list)
 
+    @unittest.skip("ANY type does not work on Mac.")
     def test_query_any(self):
         self.result, self.errorno = None, None
         def cb(result, errorno):
@@ -324,6 +461,15 @@ class DNSTest(unittest.TestCase):
         self.assertEqual(self.result, None)
         self.assertEqual(self.errorno, pycares.errno.ARES_ETIMEOUT)
 
+    def test_query_onion(self):
+        self.result, self.errorno = None, None
+        def cb(result, errorno):
+            self.result, self.errorno = result, errorno
+        self.channel.query('foo.onion', pycares.QUERY_TYPE_A, cb)
+        self.wait()
+        self.assertEqual(self.result, None)
+        self.assertEqual(self.errorno, pycares.errno.ARES_ENOTFOUND)
+
     def test_channel_nameservers(self):
         self.result, self.errorno = None, None
         def cb(result, errorno):
@@ -370,13 +516,6 @@ class DNSTest(unittest.TestCase):
         self.assertEqual(self.errorno, pycares.errno.ARES_ECONNREFUSED)
         self.assertRaises(ValueError, self.channel.set_local_ip, 'an invalid ip')
 
-    def test_channel_local_dev(self):
-        '''
-        Comments in c-ares say this only works for root, and ares ignores
-        errors. So we won't test it.
-        '''
-        pass
-
     def test_channel_timeout(self):
         self.result, self.errorno = None, None
         def cb(result, errorno):
@@ -394,19 +533,9 @@ class DNSTest(unittest.TestCase):
         from pycares.errno import ARES_SUCCESS
         self.assertTrue(True)
 
+    # FIXME
+    @unittest.skip("The site used for this test no longer returns a non-ascii SOA.")
     def test_result_not_ascii(self):
-        self.result, self.errorno = None, None
-        def cb(result, errorno):
-            self.result, self.errorno = result, errorno
-        self.channel.query('xn--cardeosapeluqueros-r0b.com', pycares.QUERY_TYPE_MX, cb)
-        self.wait()
-        self.assertNoError(self.errorno)
-        for r in self.result:
-            self.assertEqual(type(r), pycares.ares_query_mx_result)
-            self.assertIsInstance(r.host, bytes)  # it's not ASCII
-            self.assertTrue(r.ttl >= 0)
-
-    def test_result_not_ascii2(self):
         self.result, self.errorno = None, None
         def cb(result, errorno):
             self.result, self.errorno = result, errorno
@@ -415,7 +544,6 @@ class DNSTest(unittest.TestCase):
         self.assertNoError(self.errorno)
         self.assertEqual(type(self.result), pycares.ares_query_soa_result)
         self.assertIsInstance(self.result.hostmaster, bytes)  # it's not ASCII
-        self.assertTrue(self.result.ttl >= 0)
 
     def test_idna_encoding(self):
         host = 'españa.icom.museum'
@@ -425,7 +553,7 @@ class DNSTest(unittest.TestCase):
         # try encoding it as utf-8
         self.channel.gethostbyname(host.encode(), socket.AF_INET, cb)
         self.wait()
-        self.assertEqual(self.errorno, pycares.errno.ARES_ENOTFOUND)
+        self.assertNotEqual(self.errorno, None)
         self.assertEqual(self.result, None)
         # use it as is (it's IDNA encoded internally)
         self.channel.gethostbyname(host, socket.AF_INET, cb)
@@ -494,6 +622,44 @@ class DNSTest(unittest.TestCase):
             self.assertEqual(type(r), pycares.ares_query_a_result)
             self.assertNotEqual(r.host, None)
 
+    def test_lookup(self):
+        channel = pycares.Channel(
+            lookups="b",
+            timeout=1,
+            tries=1,
+            socket_receive_buffer_size=4096,
+            servers=["8.8.8.8", "8.8.4.4"],
+            tcp_port=53,
+            udp_port=53,
+            rotate=True,
+        )
+
+        def on_result(result, errorno):
+            self.result, self.errorno = result, errorno
+
+        for domain in [
+            "google.com",
+            "microsoft.com",
+            "apple.com",
+            "amazon.com",
+            "baidu.com",
+            "alipay.com",
+            "tencent.com",
+        ]:
+            self.result, self.errorno = None, None
+            self.channel.query(domain, pycares.QUERY_TYPE_A, on_result)
+            self.wait()
+            self.assertNoError(self.errorno)
+            self.assertTrue(self.result is not None and len(self.result) > 0)
+            for r in self.result:
+                self.assertEqual(type(r), pycares.ares_query_a_result)
+                self.assertNotEqual(r.host, None)
+                self.assertTrue(r.type == 'A')
+
+    def test_strerror_str(self):
+        for key in pycares.errno.errorcode:
+            self.assertTrue(type(pycares.errno.strerror(key)), str)
+
 
 if __name__ == '__main__':
     unittest.main(verbosity=2)
